@@ -1,9 +1,9 @@
 ;;; helm-core-pkg.el --- define helm-core for package.el
 
-(define-package "helm-core" "2.5.0"
+(define-package "helm-core" "2.7.1"
   "Development files for Helm"
   '((emacs "24.4")
-    (async "1.9"))
+    (async "1.9.2"))
   :url "https://emacs-helm.github.io/helm/")
 
 ;; Local Variables:
