@@ -226,6 +226,15 @@ On a symlinked directory a prefix arg will allow expanding to its true name.
 Note: The tree is reinitialized each time you enter a new tree with `C-j'
 or by entering some pattern in prompt.
 
+**** RET behavior
+
+Behave differently depending of `helm-selection' (current candidate in helm-buffer):
+
+- candidate basename is \".\"   => open it in dired.
+- candidate is a directory    => expand it.
+- candidate is a file         => open it.
+- marked candidates (1+)      => open them with default action.
+
 *** Find file at point
 
 Helm is using `ffap' partially or completely to find file at point
@@ -355,6 +364,12 @@ the command with `helm-locate-recursive-dirs-command'.
 Because this completion use an index, you may not have all the recent additions
 of directories until you update your index (with `updatedb' for locate).
 
+If for some reason you cannot use an index the find command from findutils can be
+used for this, it will be slower of course, you will have to pass the basedir as
+first argument of find and the subdir as the value for '-(i)regex' or '-(i)name'
+with the two format specs that are mandatory in `helm-locate-recursive-dirs-command',
+e.g \"find %s -type d -name '*%s*'\" or \"find %s -type d -regex .*%s.*$\".
+
 *** Insert filename at point or complete filename at point
 
 On insertion (no completion, i.e nothing at point):
@@ -455,11 +470,125 @@ You can bookmark your `helm-find-files' session with `C-x r m'.
 You can retrieve later these bookmarks easily by using M-x helm-filtered-bookmarks
 or from the current `helm-find-files' session just hitting `C-x r b'.
 
-*** Run Gid from `helm-find-files'
+*** Grep files from `helm-find-files'
 
-You can navigate to a project containing an ID file created with the `mkid'
-command from id-utils, and run the `gid' command which will use the symbol at point
-in `helm-current-buffer' as default.
+You can grep individual files from `helm-find-files' by using
+\`\\<helm-find-files-map>\\[helm-ff-run-grep]'.  This same command can
+grep also recursively files from current directory when called with a
+prefix arg, you will be prompted in this case for the file extensions
+to use (grep backend) or the types of files to use (ack-grep backend),
+see the `helm-grep-default-command' documentation to setup this.
+For compressed files or archives, use zgrep with
+\`\\<helm-find-files-map>\\[helm-ff-run-zgrep]'.
+
+Otherwise you can use other recursive commands like
+\`\\<helm-find-files-map>\\[helm-ff-run-grep-ag]' or `\\<helm-find-files-map>\\[helm-ff-run-git-grep]' that are much more
+faster than using `\\<helm-find-files-map>\\[helm-ff-run-grep]' with a
+prefix arg.  See `helm-grep-ag-command' and
+`helm-grep-git-grep-command' to setup this.
+
+You can also use the gid shell command
+\`\\<helm-find-files-map>\\[helm-ff-run-gid]' from id-utils by creating
+an ID index file with the `mkid' shell command coming with the
+id-utils package.
+
+All these grep commands are using symbol at point as default pattern.
+Note that default is a different thing than input (nothing is added to
+prompt until you hit `M-n').
+
+*** Setting up aliases in eshell allows you to setup powerful customized commands
+
+Adding eshell aliases to your `eshell-aliases-file' or using the
+`alias' command from eshell allows you to create personalized commands
+not available in `helm-find-files' actions and use them from `\\<helm-find-files-map>\\[helm-ff-run-eshell-command-on-file]'.
+Example:
+You want a command to uncompress your \"*.tar.gz\" files from `helm-find-files':
+
+1) Create an alias named untargz (or whatever) in eshell with the
+command \"alias untargz tar zxvf $*\"
+
+2) Now from `helm-find-files' select your \"*.tar.gz\" file (you can
+mark files if needed) and hit `\\<helm-find-files-map>\\[helm-ff-run-eshell-command-on-file]'.
+
+Note:
+
+When using marked files with this, the meaning of prefix arg is quite
+subtil: Say you have foo, bar and baz marked, when you run the alias
+command `example' on these files with no prefix arg it will loop on
+the file list and run sequentially `example' on each file:
+
+    example foo
+    example bar
+    example baz
+
+However with a prefix arg it will apply `example' on each file:
+
+    example foo bar baz
+
+Of course the alias command should support this.
+
+*** Using Tramp with `helm-find-files' to read remote directories
+
+`helm-find-files' is working fine with tramp with however some limitations.
+
+- By default filenames are not highlighted when working on remote directories,
+this is controled by `helm-ff-tramp-not-fancy' variable, if you change this,
+expect helm becoming very slow unless your connection is super fast.
+
+- Grepping files is not very well supported when used incrementally, see above
+grep section.
+
+- Locate is not working on remote directories.
+
+**** Some reminders about Tramp syntax
+
+Not exhaustive, please read Tramp documentation.
+
+- Connect to host 192.168.0.4 as foo user:
+
+    /scp:192.168.0.4@foo:
+
+- Connect to host 192.168.0.4 as foo user with port 2222:
+
+    /scp:192.168.0.4@foo#2222:
+
+- Connect to host 192.168.0.4 as root using multihops syntax:
+
+    /ssh:192.168.0.4@foo|sudo:192.168.0.4:
+
+Note: you can also use `tramp-default-proxies-alist' when connecting often to
+some hosts.
+
+Prefer generally scp method unless using multihops (works only with ssh method)
+specially when copying large files.
+
+Note also that you have to hit once `C-j' on top of directory at first connection
+to complete your pattern in minibuffer.
+
+**** Completing host
+
+As soon as you enter the first \":\" after method e.g =/scp:\= you will
+have some completion about previously used hosts or from your =~/.ssh/config\=
+file, hitting `C-j' or `right' on a candidate will insert this host in minibuffer
+without addind the ending \":\".
+As soon the last \":\" is entered Tramp will kick in and you should see the list
+of candidates a few seconds later.
+
+When your connection fails, be sure to delete your tramp connection before retrying
+with M-x `helm-delete-tramp-connection'.
+
+**** Editing local files as root
+
+Use the sudo method:
+
+    /sudo:host: or just /sudo::
+
+*** Attach files to a mail buffer (message-mode)
+
+If you are in a message buffer, the action will appear in action menu, otherwise
+it available at any time with \\<helm-find-files-map>\\[helm-ff-run-gnus-attach-files]
+See how behave `gnus-attach-files' for more infos.
+NOTE: Even if called `gnus-attach-files' it works with mu4e and else.
 
 ** Commands
 \\<helm-find-files-map>
@@ -686,14 +815,18 @@ Just add a space between each pattern like in most helm commands.
 
 *** Important
 
-Grepping on remote file will work only with grep, not ack-grep, but it is
-anyway bad supported as tramp doesn't support multiple process running in a
-short delay (less than 5s actually) among other things,
-so I strongly advice hitting `C-!' (i.e suspend process)
+Grepping works but it is badly supported as tramp doesn't support multiple process running in a
+short delay (less than 5s actually) among other things.
+
+Helm is suspending process automatically while you are typing with a special hook, however
+you are adviced doing this manually by hitting `C-!' (i.e suspend process)
 before entering anything in pattern, and hit again `C-!' when
 your regexp is ready to send to remote process, even if helm is handling
-this by delaying each process at 5s. 
-Or even better don't use tramp at all and mount your remote file system on SSHFS.
+this by delaying each process at 5s.
+
+If your regexp is simple enough, you can though try merely to type it directly.
+
+Another solution is to not use tramp at all and mount your remote file system on SSHFS.
 
 * Helm Gid
 
@@ -1084,6 +1217,44 @@ you don't choose from there a command using helm completion.
 ** Commands
 \\<helm-kmacro-map>")
 
+;;; Kill-ring
+;;
+;;
+(defvar helm-kill-ring-help-message
+  "* Helm kill-ring
+
+** Tips
+
+You can bring any candidate on top of kill-ring by using `\\<helm-map>\\[helm-kill-selection-and-quit]'
+on it, contrarily to the regular helm command `helm-kill-selection-and-quit'
+which is bound in helm to same key, it is here killing the real value
+without the need of prefix argument.
+
+The view of truncated candidates can be toggled, see command list below.
+
+You can bind globally `M-y' to `helm-show-kill-ring' and once in the helm kill-ring session
+you can navigate for conveniency to next/previous line with `M-y' and `M-u'.
+Of course `C-n' and `C-p' are still available.
+
+It is possible to delete unwanted candidates from kill-ring.
+
+You can concatenate marked candidates and yank them in current buffer
+creating a new entry in kill-ring.
+Note: You can insert marked candidates as well with `\\<helm-map>\\[helm-copy-to-buffer]'
+but this will not push a new entry with concatenated candidates in kill-ring.
+
+** Commands
+\\<helm-kill-ring-map>
+\\[helm-next-line]\t\tNext line.
+\\[helm-previous-line]\t\tPrevious line.
+\\[helm-kill-ring-delete]\t\tDelete entry.
+\\[helm-kill-ring-run-append]\t\tYank concatenated marked candidates.
+\\[helm-kill-ring-toggle-truncated]\t\tToggle truncated view of candidate.
+\\[helm-kill-ring-kill-selection]\t\tKill real value of selection.")
+
+;;; Org headings
+;;
+;;
 (defvar helm-org-headings-help-message
   "* Helm org headings
 
