@@ -369,7 +369,8 @@ from its directory."
                          (eq major-mode 'org-agenda-mode))
                        org-directory
                        (expand-file-name org-directory))
-                  (with-current-buffer buf default-directory)))
+                  (with-current-buffer buf
+                    (expand-file-name default-directory))))
          ;; imenu (marker).
          (marker
           (or (buffer-file-name (marker-buffer (cdr sel)))
@@ -379,7 +380,7 @@ from its directory."
                   (if (and ffap-url-regexp
                            (string-match ffap-url-regexp it))
                       it (expand-file-name it))
-                default-directory))
+                (expand-file-name default-directory)))
          ((and (stringp sel) (or (file-remote-p sel)
                                  (file-exists-p sel)))
           (expand-file-name sel))
@@ -389,11 +390,11 @@ from its directory."
          ;; Occur.
          (grep-line
           (with-current-buffer (get-buffer (car grep-line))
-            (or (buffer-file-name) default-directory)))
+            (expand-file-name (or (buffer-file-name) default-directory))))
          ;; Url.
          ((and (stringp sel) ffap-url-regexp (string-match ffap-url-regexp sel)) sel)
          ;; Default.
-         (t default-preselection))))))
+         (t (expand-file-name default-preselection)))))))
 (put 'helm-quit-and-find-file 'helm-only t)
 
 (defun helm-generic-sort-fn (s1 s2)
@@ -404,7 +405,7 @@ that is sorting is done against real value of candidate."
          (reg1  (concat "\\_<" qpattern "\\_>"))
          (reg2  (concat "\\_<" qpattern))
          (reg3  helm-pattern)
-         (split (split-string helm-pattern))
+         (split (helm-mm-split-pattern helm-pattern))
          (str1  (if (consp s1) (cdr s1) s1))
          (str2  (if (consp s2) (cdr s2) s2))
          (score (lambda (str r1 r2 r3 lst)
@@ -593,50 +594,55 @@ If STRING is non--nil return instead a space separated string."
   "Highlight and underline current position"
   (let* ((start (or start (line-beginning-position)))
          (end (or end (1+ (line-end-position))))
-         (start-match (if (or (null helm-highlight-matches-around-point-max-lines)
-                              (zerop helm-highlight-matches-around-point-max-lines))
-                          start
-                        (save-excursion
-                          (forward-line
-                           (- helm-highlight-matches-around-point-max-lines))
-                          (point-at-bol))))
-         (end-match   (if (or (null helm-highlight-matches-around-point-max-lines)
-                              (zerop helm-highlight-matches-around-point-max-lines))
-                          end
-                        (save-excursion
-                          (forward-line
-                           helm-highlight-matches-around-point-max-lines)
-                          (point-at-eol))))
+         start-match end-match
          (args (list start end buf)))
+    ;; Highlight the current line.
     (if (not helm-match-line-overlay)
         (setq helm-match-line-overlay (apply 'make-overlay args))
       (apply 'move-overlay helm-match-line-overlay args))
     (overlay-put helm-match-line-overlay
                  'face (or face 'helm-selection-line))
-    (catch 'empty-line
-      (cl-loop with ov
-               for r in (helm-remove-if-match
-                         "\\`!" (split-string
-                                 ;; Needed for highlighting AG matches.
-                                 (if (with-helm-buffer
-                                       (assq 'pcre (helm-get-current-source)))
-                                     (helm--translate-pcre-to-elisp helm-input)
-                                     helm-input)))
-               do (save-excursion
-                    (goto-char start-match)
-                    (while (condition-case _err
-                               (if helm-migemo-mode
-                                   (helm-mm-migemo-forward r end-match t)
-                                 (re-search-forward r end-match t))
-                             (invalid-regexp nil))
-                      (let ((s (match-beginning 0))
-                            (e (match-end 0)))
-                        (if (= s e)
-                            (throw 'empty-line nil)
-                          (push (setq ov (make-overlay s e))
-                                helm--match-item-overlays)
-                          (overlay-put ov 'face 'helm-match-item)
-                          (overlay-put ov 'priority 1)))))))
+    ;; Now highlight matches only if we are in helm session, we are
+    ;; maybe coming from helm-grep-mode or helm-moccur-mode buffers.
+    (when helm-alive-p
+      (if (or (null helm-highlight-matches-around-point-max-lines)
+              (zerop helm-highlight-matches-around-point-max-lines))
+          (setq start-match start
+                end-match   end)
+          (setq start-match
+                (save-excursion
+                  (forward-line
+                   (- helm-highlight-matches-around-point-max-lines))
+                  (point-at-bol))
+                  end-match
+                  (save-excursion
+                    (forward-line
+                     helm-highlight-matches-around-point-max-lines)
+                    (point-at-bol))))
+      (catch 'empty-line
+        (cl-loop with ov
+                 for r in (helm-remove-if-match
+                           "\\`!" (helm-mm-split-pattern
+                                   (if (with-helm-buffer
+                                         ;; Needed for highlighting AG matches.
+                                         (assq 'pcre (helm-get-current-source)))
+                                       (helm--translate-pcre-to-elisp helm-input)
+                                       helm-input)))
+                 do (save-excursion
+                      (goto-char start-match)
+                      (while (condition-case _err
+                                 (if helm-migemo-mode
+                                     (helm-mm-migemo-forward r end-match t)
+                                     (re-search-forward r end-match t))
+                               (invalid-regexp nil))
+                        (let ((s (match-beginning 0))
+                              (e (match-end 0)))
+                          (if (= s e)
+                              (throw 'empty-line nil)
+                              (push (setq ov (make-overlay s e))
+                                    helm--match-item-overlays)
+                              (overlay-put ov 'face 'helm-match-item)
+                              (overlay-put ov 'priority 1))))))))
     (recenter)))
 
 (defun helm--translate-pcre-to-elisp (regexp)
@@ -713,7 +719,8 @@ Assume regexp is a pcre based regexp."
              (save-selected-window
                (with-helm-window
                  (helm-aif (get-text-property (point-at-bol) 'help-echo)
-                     (popup-tip (concat " " (abbreviate-file-name it))
+                     (popup-tip (concat " " (abbreviate-file-name
+                                             (replace-regexp-in-string "\n.*" "" it)))
                                 :around nil
                                 :point (save-excursion
                                          (end-of-visual-line) (point)))))))))))
